-- SLDataText Experience Copyright (c) 2008, 2009 Jeff "Taffu" Fancher <jdfancher@gmail.com> All rights reserved.
-- Version: 1.1.0 -- Rev Date: 05/25/2009

local SLDataText = LibStub("AceAddon-3.0"):GetAddon("SLDataText")
local L = LibStub("AceLocale-3.0"):GetLocale("SLDataText")
local media = LibStub("LibSharedMedia-3.0")
local justTable, strataTable, pointTable = SLDataText.just, SLDataText.strata, SLDataText.point
local db

local MODNAME = "Experience"
local Experience = SLDataText:NewModule(MODNAME, "AceEvent-3.0")

local optGetter, optSetter
do
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
        SLDataText:RefreshModule(Experience)
	end
end

local options
local function getOptions()
	if not options then options = {
        type = "group",
        name = L["Experience"],
        arg = MODNAME,
        get = optGetter,
        set = optSetter,
        args = {
            genHeader = {
                type = "header",
                name = L["GenXPSet"],
                order = 25,
            },
            enabled = {
                type = "toggle",
                name = L["Enabled"],
                desc = L["EnabledDesc"],
                get = function() return SLDataText:GetModuleEnabled(MODNAME) end,
                set = function(info, value) 
                    SLDataText:SetModuleEnabled(MODNAME, value)
                    if ( SLDataText:GetModuleEnabled(MODNAME) ) then
                        MODNAME:PLAYER_ENTERING_WORLD()
                    end
                end,
                order = 50,
            },
            hideTooltip = {
                type = "toggle",
                name = L["HideTT"],
                desc = L["HideTTDesc"],
                order = 100,
            },
            noCombatHide = {
                type = "toggle",
                name = L["SIC"],
                desc = L["SICDesc"],
                order = 150,
            },
            dispHeader = {
                type = "header",
                name = L["DispSet"],
                order = 200,
            },
            showbar = {
                type = "toggle",
                name = L["ShowBar"],
                desc = L["ShowBarDesc"],
                order = 225,
            },
            barwidth = {
                type = "range",
                name = L["BarW"],
                desc = L["BarWDesc"],
                min = 50, max = 500, step = 5,
                order = 250,
            },
            bartexture = {
                type = "select",
                name = "Bar Texture",
                desc = "Select texture used for experience/reputation bar",
                values = media:List("statusbar"),
                get = function()
                    for k, v in pairs(media:List("statusbar")) do
                        if db.bartexture == v then
                            return k
                        end
                    end
                end,
                set = function(_, statusbar)
                    local list = media:List("statusbar")
                    db.bartexture = list[statusbar]
                    SLDataText:RefreshModule(Experience)
                end,
                width = "double",
                order = 275,
            },
            useGlobalFont = {
                type = "toggle",
                name = L["UseGblFont"],
                desc = L["UseGblFontDesc"],
                order = 300,
            },
            useGlobalFontSize = {
                type = "toggle",
                name = L["UseGblFSize"],
                desc = L["UseGblFSizeDesc"],
                order = 350,
            },
            fontFace = {
                type = "select",
                name = L["Font"],
                desc = L["FontDesc"],
                disabled = function()
                    local isTrue
                    if ( db.useGlobalFont ) then isTrue = true else isTrue = false end
                    return isTrue
                end,
                values = media:List("font"),
                get = function()
                    for k, v in pairs(media:List("font")) do
                        if db.fontFace == v then
                            return k
                        end
                    end
                end,
                set = function(_, font)
                    local list = media:List("font")
                    db.fontFace = list[font]
                    SLDataText:RefreshModule(Experience)
                end,
                width = "double",
                order = 600,
            },
            fontSize = {
                type = "range",
                name = L["FontSize"],
                desc = L["FontSizeDesc"],
                disabled = function()
                    local isTrue
                    if ( db.useGlobalFontSize ) then isTrue = true else isTrue = false end
                    return isTrue
                end,
                min = 6, max = 36, step = 1,
                width = "double",
                order = 650,
            },
            posHeader = {
                type = "header",
                name = L["LaySet"],
                order = 700,
            },
            justify = {
                type = "select",
                name = L["TextJust"],
                desc = L["TextJustDesc"],
                values = justTable,
                width = "double",
                order = 750,
            },
            anchor = {
                type = "input",
                name = L["ParFrm"],
                desc = L["ParFrmDesc"],
                get = function() return db.anchor end,
                width = "double",
                order = 800,
            },
            anchorFrom = {
                type = "select",
                name = L["AnchFrom"],
                desc = L["AnchFromDesc"],
                values = pointTable,
                get = function() return db.anchorFrom end,
                width = "double",
                order = 900,
            },
            offX = {
                type = "input",
                name = L["XOff"],
                desc = L["XOffDesc"],
                get = function() return tostring(db.offX) end,
                width = "double",
                order = 1000,
            },
            offY = {
                type = "input",
                name = L["YOff"],
                desc = L["YOffDesc"],
                get = function() return tostring(db.offY) end,
                width = "double",
                order = 1100,
            },
            strata = {
                type = "select",
                name = L["Strata"],
                desc = L["StrataDesc"],
                values = strataTable,
                width = "double",
                order = 1600,
            },
        },
    }
	end
	
	return options
end

local standingID = {
    [0] = L["Unknown"],
    [1] = L["Hated"],
    [2] = L["Hostile"],
    [3] = L["Unfriendly"],
    [4] = L["Neutral"],
    [5] = L["Friendly"],
    [6] = L["Honored"],
    [7] = L["Revered"],
    [8] = L["Exalted"],
}

local int = 1
local function buildModule(self)
    if ( not self.frame ) then self.frame = CreateFrame("FRAME", "SLDT_Experience", UIParent) end -- The frame
    -- Bar Base
    if ( not self.barframe ) then self.barframe = CreateFrame("Frame", nil, self.frame) end
    self.barframe:SetWidth(175)
    self.barframe:SetHeight(16)
    self.barframe:SetBackdrop({  bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, insets = { left = 0, top = 0, right = 0, bottom = 0 }, })
    self.barframe:SetBackdropColor(0, 0, 0, 0.4)
    self.barframe:SetFrameStrata("BACKGROUND")
    
    -- Bar    
    if ( not self.bar1 ) then self.bar1 = CreateFrame("StatusBar", nil, self.barframe) end -- Main Bar
    self.bar1:SetAllPoints(self.barframe)
    local bartex = media:Fetch("statusbar", db.bartexture)
    self.bar1:SetStatusBarTexture(bartex)
    self.bar1:SetStatusBarColor(0.1, 0.2, 1.0, 1.0)
    self.bar1:EnableMouse(false)
    
    if ( not self.rtexture ) then self.rtexture = self.bar1:CreateTexture(nil, "OVERLAY") end
    self.rtexture:SetTexture("Interface\\Buttons\\WHITE8X8")
    self.rtexture:SetWidth(2)
    self.rtexture:SetHeight(10)
    self.rtexture:SetVertexColor(1,1,1,1)
    
    -- Set scripts/etc.
    if ( not self.string ) then self.string = self.bar1:CreateFontString(nil, "OVERLAY") end -- The font string
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function Experience:PLAYER_ENTERING_WORLD()
    local enabled = SLDataText:GetModuleEnabled(MODNAME)
    if ( enabled ) then
        -- stuff you want to do only on enable
    end
    SLDataText:RefreshModule(self)
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function Experience:OnInitialize()
	self.db = SLDataText.db:RegisterNamespace(MODNAME)
    -- Register your modules default settings
    self.db:RegisterDefaults({
        profile = {
            showbar = true,
            barwidth = 200,
            bartexture = "Blizzard",
            hideTooltip = false,
            noCombatHide = false,
            useGlobalFont = true,
            fontFace = "Arial Narrow",
            useGlobalFontSize = true,
            fontSize = 12,
            justify = "CENTER",
            anchorPoint = "CENTER", 
            anchor = "UIParent", 
            anchorFrom = "CENTER", 
            offX = 0, 
            offY = 0,
            strata = "BACKGROUND",
        },
    })
	db = self.db.profile
    
    if ( not self.isMoving ) then self.isMoving = false end
	self:SetEnabledState(SLDataText:GetModuleEnabled(MODNAME))
	SLDataText:RegisterModuleOptions(MODNAME, getOptions)
end

function Experience:OnEnable()
    -- Register any events, and hide elements you don't want shown
    buildModule(self)
    self:RegisterEvent("PLAYER_XP_UPDATE", "Refresh")
    self:RegisterEvent("PLAYER_LEVEL_UP", "Refresh")
    self:RegisterEvent("UPDATE_EXHAUSTION", "Refresh")
    self:RegisterEvent("PLAYER_UPDATE_RESTING", "Refresh")
    if ( not self.frame:IsShown() ) then self.frame:Show() end
end

function Experience:OnDisable()
    -- Unregister any events, nil scripts, and show elements you've hidden
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
    self:UnregisterEvent("PLAYER_XP_UPDATE", "Refresh")
    self:UnregisterEvent("PLAYER_LEVEL_UP", "Refresh")
    self:UnregisterEvent("UPDATE_EXHAUSTION", "Refresh")
    self:UnregisterEvent("PLAYER_UPDATE_RESTING", "Refresh")
	if ( self.frame:IsShown() ) then self.frame:Hide() end
end

local function checkLevel()
    local level = UnitLevel("player")
    local isMax = false
    if ( level == 80 ) then
        isMax = true
    end
    return isMax
end

-- Main update, used to refresh your modules data
function Experience:Refresh()
    -- Gather your data
    local lvl = UnitLevel("player")
    local name = select(1, GetWatchedFactionInfo())
    local curXP, maxXP, restXP, curXPPerc, restXPPerc
    local width, step, restPos, standing
    
    if ( name ~= nil ) then
        if ( not self.frame:IsShown() ) then self.frame:Show() end
        local name, react, min, max, value = GetWatchedFactionInfo()
        local curRep, maxRep = abs(value-min), abs(max-min)
        curXP = curRep
        maxXP = maxRep
        curXPPerc = (curXP/maxXP)*100
        -- Set bar values
        self.bar1:SetMinMaxValues(0, maxXP)
        self.bar1:SetValue(curXP)
        -- Adjustments
        width = self.barframe:GetWidth()
        step = (width/maxXP)
        restPos = 0
        standing = standingID[react]
    elseif ( name == nil and lvl < 80 ) then
        if ( not self.frame:IsShown() ) then self.frame:Show() end
        curXP, maxXP, restXP = UnitXP("player"), UnitXPMax("player"), GetXPExhaustion() or 0
        curXPPerc = (curXP/maxXP)*100
        -- Set bar values
        self.bar1:SetMinMaxValues(0, maxXP)
        self.bar1:SetValue(curXP)
        -- Adjustments
        width = self.barframe:GetWidth()
        step = (width/maxXP)
        restPos = floor(curXP*step)+floor(restXP*step)
        restXPPerc = ((restXP/maxXP)*100)
    elseif ( name == nil and lvl == 80 ) then
        if ( self.frame:IsShown() ) then self.frame:Hide() end
    end
    
    if ( restXP and restXP > 0 and (curXP+restXP) <= maxXP ) then -- If theres rest XP but not enough to overflow
        self.rtexture:ClearAllPoints()
        self.rtexture:SetPoint("BOTTOMLEFT", restPos, 0)
    elseif ( restXP == 0 or restXP == nil ) then
        self.rtexture:Hide()
    else
        self.rtexture:ClearAllPoints()
        self.rtexture:SetPoint("BOTTOMRIGHT", 0, 0)
        self.rtexture:Show()
    end
    
    -- Here we fetch the color, determine any display options, and set the value of the module data
    local color = SLDataText.db.profile.gColor
    if ( lvl < 80 ) then
        if ( name ~= nil ) then -- If tracking a reputation
            self.string:SetFormattedText("|cff%s%.0f%%|r", color, curXPPerc)
            if ( SLDataText.db.profile.locked and not db.hideTooltip ) then
                self.frame:EnableMouse(true)
                self.frame:SetScript("OnEnter", function(self)
                    GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
                    GameTooltip:AddLine("|cffffff00" .. name .."|r")
                    GameTooltip:AddLine("|cffffff00" .. standing .."|r")
                    GameTooltip:AddLine(" ", 1, 1, 1)
                    GameTooltip:AddDoubleLine(L["Current Rep:"], format("%.1f%%", curXPPerc), 1, 1, 1, 1, 1, 1)
                    GameTooltip:AddDoubleLine(L["Rep To Go:"], format("%.1f%%", abs(100 - curXPPerc)), 1, 1, 1, 1, 1, 1)
                    GameTooltip:Show()
                end)
                self.frame:SetScript("OnLeave", function()
                    if ( GameTooltip:IsShown() ) then
                        GameTooltip:Hide()
                    end
                end)
            else
                self.frame:EnableMouse(false)
                self.frame:SetScript("OnEnter", nil)
                self.frame:SetScript("OnLeave", nil)
            end
        else
            self.string:SetFormattedText("|cff%s%.0f%%|r", color, curXPPerc)
            if ( SLDataText.db.profile.locked and not db.hideTooltip ) then
                self.frame:EnableMouse(true)
                self.frame:SetScript("OnEnter", function(self)
                    GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
                    GameTooltip:AddLine("|cffffff00" .. L["ExpData"] .."|r")
                    GameTooltip:AddLine(" ", 1, 1, 1)
                    GameTooltip:AddDoubleLine(L["Current XP:"], format("%.1f%%", curXPPerc), 1, 1, 1, 1, 1, 1)
                    GameTooltip:AddDoubleLine(L["XP To Go:"], format("%.1f%%", abs(100 - curXPPerc)), 1, 1, 1, 1, 1, 1)
                    if ( GetXPExhaustion() ) then
                        GameTooltip:AddDoubleLine(L["Rested XP:"], format("%.1f%%", restXPPerc), 1, 1, 1, 1, 1, 1)
                    end
                    GameTooltip:Show()
                end)
                self.frame:SetScript("OnLeave", function()
                    if ( GameTooltip:IsShown() ) then
                        GameTooltip:Hide()
                    end
                end)
            else
                self.frame:EnableMouse(false)
                self.frame:SetScript("OnEnter", nil)
                self.frame:SetScript("OnLeave", nil)
            end
        end
        
        local parent = self.barframe:GetParent()
        local offset
        if ( db.justify == "RIGHT" ) then offset = 4 elseif ( db.justify == "LEFT" ) then offset = -4 end    
        self.barframe:ClearAllPoints()
        self.barframe:SetPoint(db.justify, parent, db.justify, offset, 0)
        self.barframe:SetHeight(self.string:GetHeight())
        self.barframe:SetWidth(db.barwidth)
        self.rtexture:SetHeight(abs(self.string:GetHeight()*.8))
        if ( not db.showbar ) then
            self.string:SetParent(self.frame)
            if ( self.barframe:IsShown() ) then self.barframe:Hide() end
        else
            self.string:SetParent(self.bar1)
            if ( not self.barframe:IsShown() ) then self.barframe:Show() end
        end
    else
        if ( name ~= nil ) then
            self.string:SetFormattedText("|cff%s%.0f%%|r", color, abs((curXP/maxXP)*100))            
            if ( SLDataText.db.profile.locked and not db.hideTooltip ) then
                self.frame:EnableMouse(true)
                self.frame:SetScript("OnEnter", function(self)
                    GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
                    GameTooltip:AddLine("|cffffff00" .. name .."|r")
                    GameTooltip:AddLine("|cffffff00" .. standing .."|r")
                    GameTooltip:AddLine(" ", 1, 1, 1)
                    GameTooltip:AddDoubleLine(L["Current Rep:"], format("%.1f%%", curXPPerc), 1, 1, 1, 1, 1, 1)
                    GameTooltip:AddDoubleLine(L["Rep To Go:"], format("%.1f%%", abs(100 - curXPPerc)), 1, 1, 1, 1, 1, 1)
                    GameTooltip:Show()
                end)
                self.frame:SetScript("OnLeave", function()
                    if ( GameTooltip:IsShown() ) then
                        GameTooltip:Hide()
                    end
                end)
                self.frame:SetScript("OnLeave", function()
                    if ( GameTooltip:IsShown() ) then
                        GameTooltip:Hide()
                    end
                end)
            else
                self.frame:EnableMouse(false)
                self.frame:SetScript("OnEnter", nil)
                self.frame:SetScript("OnLeave", nil)
            end
            local parent = self.barframe:GetParent()
            local offset
            if ( db.justify == "RIGHT" ) then offset = 4 elseif ( db.justify == "LEFT" ) then offset = -4 end    
            self.barframe:ClearAllPoints()
            self.barframe:SetPoint(db.justify, parent, db.justify, offset, 0)
            self.barframe:SetHeight(self.string:GetHeight())
            self.barframe:SetWidth(db.barwidth)
            self.rtexture:SetHeight(abs(self.string:GetHeight()*.8))
            if ( not self.frame:IsShown() ) then self.frame:Show() end
            if ( not db.showbar ) then
                self.string:SetParent(self.frame)
                if ( self.barframe:IsShown() ) then self.barframe:Hide() end
            else
                self.string:SetParent(self.bar1)
                if ( not self.barframe:IsShown() ) then self.barframe:Show() end
            end
        end
    end
    
    -- And then update the module for refreshing/resizing text/frame
    local bartex = media:Fetch("statusbar", db.bartexture)
    self.bar1:SetStatusBarTexture(bartex)
    SLDataText:UpdateModule(self)
end

function ReputationWatchBar_Update(newLevel)
	local name, reaction, min, max, value = GetWatchedFactionInfo();
	local visibilityChanged = nil;
	if ( not newLevel ) then
		newLevel = UnitLevel("player");
	end
	if ( name ) then
		-- See if it was already shown or not
		if ( not ReputationWatchBar:IsShown() ) then
			visibilityChanged = 1;
		end
 
		-- Normalize values
		max = max - min;
		value = value - min;
		min = 0;
		ReputationWatchStatusBar:SetMinMaxValues(min, max);
		ReputationWatchStatusBar:SetValue(value);
		ReputationWatchStatusBarText:SetText(name.." "..value.." / "..max);
		local color = FACTION_BAR_COLORS[reaction];
		ReputationWatchStatusBar:SetStatusBarColor(color.r, color.g, color.b);
		ReputationWatchBar:Show();
 
		-- If the player is max level then replace the xp bar with the watched reputation, otherwise stack the reputation watch bar on top of the xp bar
		ReputationWatchStatusBar:SetFrameLevel(MainMenuBarArtFrame:GetFrameLevel()-1);
		if ( newLevel < MAX_PLAYER_LEVEL ) then
			-- Reconfigure reputation bar
			ReputationWatchStatusBar:SetHeight(8);
			ReputationWatchBar:ClearAllPoints();
			ReputationWatchBar:SetPoint("BOTTOM", MainMenuBar, "TOP", 0, -3);
			ReputationWatchStatusBarText:SetPoint("CENTER", ReputationWatchBarOverlayFrame, "CENTER", 0, 3);
			ReputationWatchBarTexture0:Show();
			ReputationWatchBarTexture1:Show();
			ReputationWatchBarTexture2:Show();
			ReputationWatchBarTexture3:Show();
 
			ReputationXPBarTexture0:Hide();
			ReputationXPBarTexture1:Hide();
			ReputationXPBarTexture2:Hide();
			ReputationXPBarTexture3:Hide();
 
			-- Show the XP bar
			MainMenuExpBar:Show();
			MainMenuExpBar.pauseUpdates = nil;
			-- Hide max level bar
			MainMenuBarMaxLevelBar:Hide();
		else
			-- Replace xp bar
			ReputationWatchStatusBar:SetHeight(13);
			ReputationWatchBar:ClearAllPoints();
			ReputationWatchBar:SetPoint("TOP", MainMenuBar, "TOP", 0, 0);
			ReputationWatchStatusBarText:SetPoint("CENTER", ReputationWatchBarOverlayFrame, "CENTER", 0, 1);
			ReputationWatchBarTexture0:Hide();
			ReputationWatchBarTexture1:Hide();
			ReputationWatchBarTexture2:Hide();
			ReputationWatchBarTexture3:Hide();
 
			ReputationXPBarTexture0:Show();
			ReputationXPBarTexture1:Show();
			ReputationXPBarTexture2:Show();
			ReputationXPBarTexture3:Show();
 
			ExhaustionTick:Hide();
 
			-- Hide the XP bar
			MainMenuExpBar:Hide();
			MainMenuExpBar.pauseUpdates = true;
			-- Hide max level bar
			MainMenuBarMaxLevelBar:Hide();
		end
 
	else
		if ( ReputationWatchBar:IsShown() ) then
			visibilityChanged = 1;
		end
		ReputationWatchBar:Hide();
		if ( newLevel == MAX_PLAYER_LEVEL ) then
			MainMenuExpBar:Hide();
			MainMenuExpBar.pauseUpdates = true;
			MainMenuBarMaxLevelBar:Show();
			ExhaustionTick:Hide();
		else
			MainMenuExpBar:Show();
			MainMenuExpBar.pauseUpdates = nil;
			MainMenuBarMaxLevelBar:Hide();
		end
	end
	if ( visibilityChanged ) then
		UIParent_ManageFramePositions();
		updateContainerFrameAnchors();
	end
    
    if ( Experience.frame ) then
        Experience:Refresh()
    end
end
